@@ -39,6 +39,8 @@
 #include <string.h>
 #include <stdlib.h>
 
+#include "json-types-private.h"
+
 #include "json-gobject.h"
 #include "json-parser.h"
 #include "json-generator.h"
@@ -189,7 +191,7 @@ json_deserialize_pspec (GValue     *value,
         {
           JsonArray *array = json_node_get_array (node);
           guint i, array_len = json_array_get_length (array);
-          GPtrArray *str_array = g_ptr_array_sized_new (array_len);
+          GPtrArray *str_array = g_ptr_array_sized_new (array_len + 1);
 
           for (i = 0; i < array_len; i++)
             {
@@ -202,9 +204,11 @@ json_deserialize_pspec (GValue     *value,
                 g_ptr_array_add (str_array, (gpointer) json_node_get_string (val));
             }
 
+          g_ptr_array_add (str_array, NULL);
+
           g_value_set_boxed (value, str_array->pdata);
 
-          g_ptr_array_free (str_array, FALSE);
+          g_ptr_array_free (str_array, TRUE);
 
           retval = TRUE;
         }
@@ -513,7 +517,8 @@ json_serializable_get_type (void)
 
   if (!iface_type)
     iface_type =
-      g_type_register_static_simple (G_TYPE_INTERFACE, "JsonSerializable",
+      g_type_register_static_simple (G_TYPE_INTERFACE,
+                                     g_intern_static_string ("JsonSerializable"),
                                      sizeof (JsonSerializableIface),
                                      json_serializable_base_init,
                                      0, NULL, 0);
@@ -721,7 +726,7 @@ json_serialize_gobject (GObject *gobject,
         node = json_serialize_pspec (&value, pspec);
 
       if (node)
-        json_object_add_member (object, pspec->name, node);
+        json_object_set_member (object, pspec->name, node);
 
       g_value_unset (&value);
     }
